require "spec_helper"

describe TweekMetrics::Handler do
  let(:statsd) { double("Statsd") }

  class Fake
    def self.execute
    end
  end

  before do
    TweekMetrics.stub(:statsd).and_return(statsd)
    statsd.stub(:increment)
    statsd.stub(:timing)
  end

  context "measuring counter" do
    it "increments the global counter" do
      TweekMetrics.measure([:counter, :time], [:request]) { Fake.execute }
      expect(statsd).to have_received(:increment).with("#{$API_NAME}.global.request")
    end

    it "increments the client specific counter" do
      TweekMetrics.measure([:counter, :time], [:request]) { Fake.execute }
      expect(statsd).to have_received(:increment).with("#{$API_NAME}.#{$CLIENT_NAME}.request")
    end
  end

  context "measuring timing" do
    it "adds a new value to the global timing" do
      TweekMetrics.measure([:counter, :time], [:request]) { Fake.execute }
      expect(statsd).to have_received(:timing).with("#{$API_NAME}.global.request", kind_of(Numeric))
    end

    it "adds a new value to the client specific timing" do
      TweekMetrics.measure([:counter, :time], [:request]) { Fake.execute }
      expect(statsd).to have_received(:timing).with("#{$API_NAME}.#{$CLIENT_NAME}.request", kind_of(Numeric))
    end
  end

  context "exception handler" do
    before do
      expect(Fake).to receive(:execute).and_raise(ArgumentError.new)
    end

    it "increments the global exception counter" do
      TweekMetrics.measure([:counter, :time], [:request]) { Fake.execute }
      expect(statsd).to have_received(:increment).with("#{$API_NAME}.global.exception.ArgumentError")
    end

    it "increments the client exception counter" do
      TweekMetrics.measure([:counter, :time], [:request]) { Fake.execute }
      expect(statsd).to have_received(:increment).with("#{$API_NAME}.#{$CLIENT_NAME}.exception.ArgumentError")
    end
  end
end